@@ -57,6 +57,13 @@ class DFFormat(object):
         self.msg_types = msg_types
         self.msg_mults = msg_mults
 
+def null_term(str):
+    '''null terminate a string'''
+    idx = str.find("\0")
+    if idx != -1:
+        str = str[:idx]
+    return str
+
 class DFMessage(object):
     def __init__(self, fmt, elements, apply_multiplier):
         self._d = {}
@@ -68,7 +75,8 @@ class DFMessage(object):
             if fmt.format[i] != 'M' or apply_multiplier:
                 self._d[name] = fmt.msg_types[i](self._d[name])
             if fmt.msg_types[i] == str:
-                self._d[name] = self._d[name].rstrip('\0')
+                self._d[name] = self._d[name]
+                self._d[name] = null_term(self._d[name])
             if mul is not None and apply_multiplier:
                 self._d[name] = self._d[name] * mul
         self._fieldnames = fmt.columns
@@ -111,7 +119,7 @@ class DFReader(object):
     def _find_time_base_new(self, gps):
         '''work out time basis for the log - new style'''
         t = self._gpsTimeToTime(gps.Week, gps.TimeMS*0.001)
-        self.timebase = t - gps.T
+        self.timebase = t - gps.T*0.001
         self.new_timestamps = True
 
     def _find_time_base_px4(self, gps):
@@ -123,7 +131,7 @@ class DFReader(object):
     def _find_time_base(self):
         '''work out time basis for the log'''
         self.timebase = 0
-        gps1 = self.recv_match(type='GPS', condition='getattr(GPS,"Week",0)!=0 or getattr(GPS,"GPSTime")!=0')
+        gps1 = self.recv_match(type='GPS', condition='getattr(GPS,"Week",0)!=0 or getattr(GPS,"GPSTime",0)!=0')
         if gps1 is None:
             self._rewind()
             return
@@ -217,8 +225,10 @@ class DFReader(object):
         if type == 'MODE':
             if isinstance(m.Mode, str):
                 self.flightmode = m.Mode.upper()
-            else:
+            elif 'ModeNum' in m._fieldnames:
                 self.flightmode = mavutil.mode_string_apm(m.ModeNum)
+            else:
+                self.flightmode = mavutil.mode_string_acm(m.Mode)
         self._set_time(m)
 
     def recv_match(self, condition=None, type=None, blocking=False):
@@ -255,6 +265,7 @@ class DFReader_binary(DFReader):
         }
         self._rewind()
         self._find_time_base()
+        self._rewind()
 
     def _rewind(self):
         '''rewind to start of log'''
@@ -269,7 +280,7 @@ class DFReader_binary(DFReader):
             
         hdr = self.data[self.offset:self.offset+3]
         if (ord(hdr[0]) != self.HEAD1 or ord(hdr[1]) != self.HEAD2):
-            raise Exception("Invalid header: %02X %02X, must be %02X %02X" % (ord(hdr[0]), ord(hdr[1]), self.HEAD1, self.HEAD2))
+            return None
         msg_type = ord(hdr[2])
 
         self.offset += 3
@@ -287,12 +298,12 @@ class DFReader_binary(DFReader):
         except Exception:
             print("Failed to parse %s/%s with len %u (remaining %u)" % (fmt.name, fmt.msg_struct, len(body), self.remaining))
             raise
-        name = fmt.name.rstrip('\0')
+        name = null_term(fmt.name)
         if name == 'FMT':
             # add to formats
             # name, len, format, headings
-            self.formats[elements[0]] = DFFormat(elements[2].rstrip('\0'), elements[1],
-                                                 elements[3].rstrip('\0'), elements[4].rstrip('\0'))
+            self.formats[elements[0]] = DFFormat(null_term(elements[2]), elements[1],
+                                                 null_term(elements[3]), null_term(elements[4]))
 
         self.offset += fmt.len-3
         self.remaining -= fmt.len-3
